@@ -31,6 +31,7 @@ from gi.repository import Gio, GLib
 from apport.packaging_impl import impl as packaging
 
 import apport
+import apport.fileutils
 
 try:
     _path_key_trans = ''.maketrans('#/-_+ ', '....._')
@@ -756,43 +757,6 @@ def package_versions(*packages):
     return '\n'.join([fmt % v for v in versions])
 
 
-def shared_libraries(path):
-    '''Returns a list of strings containing the sonames of shared libraries
-    with which the specified binary is linked.'''
-
-    libs = set()
-
-    for line in command_output(['ldd', path]).split('\n'):
-        try:
-            lib, rest = line.split('=>', 1)
-        except ValueError:
-            continue
-
-        lib = lib.strip()
-        libs.add(lib)
-
-    return libs
-
-
-def links_with_shared_library(path, lib):
-    '''Returns True if the binary at path links with the library named lib.
-
-    path should be a fully qualified path (e.g. report['ExecutablePath'])
-    lib may be of the form 'lib<name>' or 'lib<name>.so.<version>'
-    '''
-
-    libs = shared_libraries(path)
-
-    if lib in libs:
-        return True
-
-    for linked_lib in libs:
-        if linked_lib.startswith(lib + '.so.'):
-            return True
-
-    return False
-
-
 def _get_module_license(module):
     '''Return the license for a given kernel module.'''
 
@@ -926,3 +890,8 @@ def attach_default_grub(report, key=None):
                         else '### PASSWORD LINE REMOVED ###'
                         for l in f.readlines()]
             report[key] = ''.join(filtered)
+
+
+# backwards compatible API
+shared_libraries = apport.fileutils.shared_libraries
+links_with_shared_library = apport.fileutils.links_with_shared_library
